@@ -43,14 +43,19 @@
 #include "nsEventListenerManager.h"
 #include "nsContentUtils.h"
 #include "nsDOMError.h"
+#include "mozilla/FunctionTimer.h"
 #include "nsMutationEvent.h"
 #include NEW_H
 #include "nsFixedSizeAllocator.h"
 #include "nsINode.h"
 #include "nsPIDOMWindow.h"
+#include "nsDOMPopStateEvent.h"
 
 #define NS_TARGET_CHAIN_FORCE_CONTENT_DISPATCH  (1 << 0)
 #define NS_TARGET_CHAIN_WANTS_WILL_HANDLE_EVENT (1 << 1)
+#define NS_TARGET_CHAIN_MAY_HAVE_MANAGER        (1 << 2)
+
+static nsEventTargetChainItem* gCachedETCI = nsnull;
 
 // nsEventTargetChainItem represents a single item in the event target chain.
 class nsEventTargetChainItem
@@ -64,7 +69,13 @@ public:
                                         nsPIDOMEventTarget* aTarget,
                                         nsEventTargetChainItem* aChild = nsnull)
   {
-    void* place = aAllocator->Alloc(sizeof(nsEventTargetChainItem));
+    void* place = nsnull;
+    if (gCachedETCI) {
+      place = gCachedETCI;
+      gCachedETCI = gCachedETCI->mNext;
+    } else {
+      place = aAllocator->Alloc(sizeof(nsEventTargetChainItem));
+    }
     return place
       ? ::new (place) nsEventTargetChainItem(aTarget, aChild)
       : nsnull;
@@ -82,7 +93,8 @@ public:
     while (item) {
       nsEventTargetChainItem* parent = item->mParent;
       item->~nsEventTargetChainItem();
-      aAllocator->Free(item, sizeof(nsEventTargetChainItem));
+      item->mNext = gCachedETCI;
+      gCachedETCI = item;
       --sCurrentEtciCount;
       item = parent;
     }
@@ -94,7 +106,7 @@ public:
     return !!(mTarget);
   }
 
-  nsISupports* GetNewTarget()
+  nsPIDOMEventTarget* GetNewTarget()
   {
     return mNewTarget;
   }
@@ -118,7 +130,6 @@ public:
     return !!(mFlags & NS_TARGET_CHAIN_FORCE_CONTENT_DISPATCH);
   }
 
-
   void SetWantsWillHandleEvent(PRBool aWants)
   {
     if (aWants) {
@@ -133,6 +144,20 @@ public:
     return !!(mFlags & NS_TARGET_CHAIN_WANTS_WILL_HANDLE_EVENT);
   }
 
+  void SetMayHaveListenerManager(PRBool aMayHave)
+  {
+    if (aMayHave) {
+      mFlags |= NS_TARGET_CHAIN_MAY_HAVE_MANAGER;
+    } else {
+      mFlags &= ~NS_TARGET_CHAIN_MAY_HAVE_MANAGER;
+    }
+  }
+
+  PRBool MayHaveListenerManager()
+  {
+    return !!(mFlags & NS_TARGET_CHAIN_MAY_HAVE_MANAGER);
+  }
+  
   nsPIDOMEventTarget* CurrentTarget()
   {
     return mTarget;
@@ -147,7 +172,8 @@ public:
   nsresult HandleEventTargetChain(nsEventChainPostVisitor& aVisitor,
                                   PRUint32 aFlags,
                                   nsDispatchingCallback* aCallback,
-                                  PRBool aMayHaveNewListenerManagers);
+                                  PRBool aMayHaveNewListenerManagers,
+                                  nsCxPusher* aPusher);
 
   /**
    * Resets aVisitor object and calls PreHandleEvent.
@@ -157,11 +183,38 @@ public:
 
   /**
    * If the current item in the event target chain has an event listener
-   * manager, this method sets the .currentTarget to the CurrentTarget()
-   * and calls nsIEventListenerManager::HandleEvent().
+   * manager, this method calls nsIEventListenerManager::HandleEvent().
    */
   nsresult HandleEvent(nsEventChainPostVisitor& aVisitor, PRUint32 aFlags,
-                       PRBool aMayHaveNewListenerManagers);
+                       PRBool aMayHaveNewListenerManagers,
+                       nsCxPusher* aPusher)
+  {
+    if (WantsWillHandleEvent()) {
+      mTarget->WillHandleEvent(aVisitor);
+    }
+    if (aVisitor.mEvent->flags & NS_EVENT_FLAG_STOP_DISPATCH) {
+      return NS_OK;
+    }
+    if (!mManager) {
+      if (!MayHaveListenerManager() && !aMayHaveNewListenerManagers) {
+        return NS_OK;
+      }
+      mManager =
+        static_cast<nsEventListenerManager*>(mTarget->GetListenerManager(PR_FALSE));
+    }
+    if (mManager) {
+      NS_ASSERTION(aVisitor.mEvent->currentTarget == nsnull,
+                   "CurrentTarget should be null!");
+      mManager->HandleEvent(aVisitor.mPresContext, aVisitor.mEvent,
+                            &aVisitor.mDOMEvent,
+                            CurrentTarget(), aFlags,
+                            &aVisitor.mEventStatus,
+                            aPusher);
+      NS_ASSERTION(aVisitor.mEvent->currentTarget == nsnull,
+                   "CurrentTarget should be null!");
+    }
+    return NS_OK;
+  }
 
   /**
    * Copies mItemFlags and mItemData to aVisitor and calls PostHandleEvent.
@@ -178,14 +231,18 @@ public:
 
   nsCOMPtr<nsPIDOMEventTarget>      mTarget;
   nsEventTargetChainItem*           mChild;
-  nsEventTargetChainItem*           mParent;
+  union {
+    nsEventTargetChainItem*         mParent;
+     // This is used only when caching ETCI objects.
+    nsEventTargetChainItem*         mNext;
+  };
   PRUint16                          mFlags;
   PRUint16                          mItemFlags;
   nsCOMPtr<nsISupports>             mItemData;
   // Event retargeting must happen whenever mNewTarget is non-null.
   nsCOMPtr<nsPIDOMEventTarget>      mNewTarget;
   // Cache mTarget's event listener manager.
-  nsCOMPtr<nsIEventListenerManager> mManager;
+  nsRefPtr<nsEventListenerManager>  mManager;
 
   static PRUint32                   sMaxEtciCount;
   static PRUint32                   sCurrentEtciCount;
@@ -215,41 +272,12 @@ nsEventTargetChainItem::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
   nsresult rv = mTarget->PreHandleEvent(aVisitor);
   SetForceContentDispatch(aVisitor.mForceContentDispatch);
   SetWantsWillHandleEvent(aVisitor.mWantsWillHandleEvent);
+  SetMayHaveListenerManager(aVisitor.mMayHaveListenerManager);
   mItemFlags = aVisitor.mItemFlags;
   mItemData = aVisitor.mItemData;
   return rv;
 }
 
-nsresult
-nsEventTargetChainItem::HandleEvent(nsEventChainPostVisitor& aVisitor,
-                                    PRUint32 aFlags,
-                                    PRBool aMayHaveNewListenerManagers)
-{
-  if (WantsWillHandleEvent()) {
-    mTarget->WillHandleEvent(aVisitor);
-  }
-  if (aVisitor.mEvent->flags & NS_EVENT_FLAG_STOP_DISPATCH) {
-    return NS_OK;
-  }
-  if (!mManager) {
-    if (!aMayHaveNewListenerManagers) {
-      return NS_OK;
-    }
-    mManager = mTarget->GetListenerManager(PR_FALSE);
-  }
-  if (mManager) {
-    NS_ASSERTION(aVisitor.mEvent->currentTarget == nsnull,
-                 "CurrentTarget should be null!");
-    mManager->HandleEvent(aVisitor.mPresContext, aVisitor.mEvent,
-                          &aVisitor.mDOMEvent,
-                          CurrentTarget(), aFlags,
-                          &aVisitor.mEventStatus);
-    NS_ASSERTION(aVisitor.mEvent->currentTarget == nsnull,
-                 "CurrentTarget should be null!");
-  }
-  return NS_OK;
-}
-
 nsresult
 nsEventTargetChainItem::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
 {
@@ -262,11 +290,12 @@ nsEventTargetChainItem::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
 nsresult
 nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor, PRUint32 aFlags,
                                                nsDispatchingCallback* aCallback,
-                                               PRBool aMayHaveNewListenerManagers)
+                                               PRBool aMayHaveNewListenerManagers,
+                                               nsCxPusher* aPusher)
 {
   PRUint32 createdELMs = nsEventListenerManager::sCreatedCount;
   // Save the target so that it can be restored later.
-  nsCOMPtr<nsISupports> firstTarget = aVisitor.mEvent->target;
+  nsCOMPtr<nsPIDOMEventTarget> firstTarget = aVisitor.mEvent->target;
 
   // Capture
   nsEventTargetChainItem* item = this;
@@ -278,14 +307,15 @@ nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor
         !(aVisitor.mEvent->flags & NS_EVENT_FLAG_STOP_DISPATCH)) {
       item->HandleEvent(aVisitor, aFlags & NS_EVENT_CAPTURE_MASK,
                         aMayHaveNewListenerManagers ||
-                        createdELMs != nsEventListenerManager::sCreatedCount);
+                        createdELMs != nsEventListenerManager::sCreatedCount,
+                        aPusher);
     }
 
     if (item->GetNewTarget()) {
       // item is at anonymous boundary. Need to retarget for the child items.
       nsEventTargetChainItem* nextTarget = item->mChild;
       while (nextTarget) {
-        nsISupports* newTarget = nextTarget->GetNewTarget();
+        nsPIDOMEventTarget* newTarget = nextTarget->GetNewTarget();
         if (newTarget) {
           aVisitor.mEvent->target = newTarget;
           break;
@@ -307,7 +337,8 @@ nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor
     //       <xul:dialog>'s buttons. Bug 235441.
     item->HandleEvent(aVisitor, aFlags,
                       aMayHaveNewListenerManagers ||
-                      createdELMs != nsEventListenerManager::sCreatedCount);
+                      createdELMs != nsEventListenerManager::sCreatedCount,
+                      aPusher);
   }
   if (aFlags & NS_EVENT_FLAG_SYSTEM_EVENT) {
     item->PostHandleEvent(aVisitor);
@@ -317,7 +348,7 @@ nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor
   aVisitor.mEvent->flags &= ~NS_EVENT_FLAG_CAPTURE;
   item = item->mParent;
   while (item) {
-    nsISupports* newTarget = item->GetNewTarget();
+    nsPIDOMEventTarget* newTarget = item->GetNewTarget();
     if (newTarget) {
       // Item is at anonymous boundary. Need to retarget for the current item
       // and for parent items.
@@ -329,7 +360,8 @@ nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor
            item->ForceContentDispatch()) &&
           !(aVisitor.mEvent->flags & NS_EVENT_FLAG_STOP_DISPATCH)) {
         item->HandleEvent(aVisitor, aFlags & NS_EVENT_BUBBLE_MASK,
-                          createdELMs != nsEventListenerManager::sCreatedCount);
+                          createdELMs != nsEventListenerManager::sCreatedCount,
+                          aPusher);
       }
       if (aFlags & NS_EVENT_FLAG_SYSTEM_EVENT) {
         item->PostHandleEvent(aVisitor);
@@ -359,7 +391,8 @@ nsEventTargetChainItem::HandleEventTargetChain(nsEventChainPostVisitor& aVisitor
     aVisitor.mEvent->target = firstTarget;
     HandleEventTargetChain(aVisitor, aFlags | NS_EVENT_FLAG_SYSTEM_EVENT,
                            aCallback,
-                           createdELMs != nsEventListenerManager::sCreatedCount);
+                           createdELMs != nsEventListenerManager::sCreatedCount,
+                           aPusher);
   }
 
   return NS_OK;
@@ -396,6 +429,7 @@ public:
     }
     if (!sEtciPoolUsers) {
       if (nsEventTargetChainItem::MaxEtciCount() > NS_CHAIN_POOL_SIZE) {
+        gCachedETCI = nsnull;
         delete sEtciPool;
         sEtciPool = nsnull;
         nsEventTargetChainItem::ResetMaxEtciCount();
@@ -406,6 +440,7 @@ public:
   static void Shutdown()
   {
     if (!sEtciPoolUsers) {
+      gCachedETCI = nsnull;
       delete sEtciPool;
       sEtciPool = nsnull;
       nsEventTargetChainItem::ResetMaxEtciCount();
@@ -437,6 +472,14 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
                  NS_ERROR_ILLEGAL_VALUE);
   NS_ASSERTION(!aTargets || !aEvent->message, "Wrong parameters!");
 
+#ifdef NS_FUNCTION_TIMER
+  const char* timer_event_name = nsDOMEvent::GetEventName(aEvent->message);
+  NS_TIME_FUNCTION_MIN_FMT(20, "Dispatching '%s' event",
+                           timer_event_name ? timer_event_name : "<other>");
+#endif
+
+  nsCOMPtr<nsPIDOMEventTarget> target = do_QueryInterface(aTarget);
+
   if (aEvent->flags & NS_EVENT_FLAG_ONLY_CHROME_DISPATCH) {
     nsCOMPtr<nsINode> node = do_QueryInterface(aTarget);
     if (!node) {
@@ -453,14 +496,26 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
       // If we can't dispatch the event to chrome, do nothing.
       NS_ENSURE_TRUE(win && win->GetChromeEventHandler(), NS_OK);
       // Set the target to be the original dispatch target,
-      aEvent->target = aTarget;
+      aEvent->target = target;
       // but use chrome event handler for event target chain.
-      aTarget = win->GetChromeEventHandler();
+      target = do_QueryInterface(win->GetChromeEventHandler());
     }
   }
 
-  nsCOMPtr<nsPIDOMEventTarget> target = do_QueryInterface(aTarget);
 #ifdef DEBUG
+  if (!nsContentUtils::IsSafeToRunScript()) {
+    nsresult rv = NS_ERROR_FAILURE;
+    if (target->GetContextForEventHandlers(&rv) ||
+        NS_FAILED(rv)) {
+      nsCOMPtr<nsINode> node = do_QueryInterface(target);
+      if (node && nsContentUtils::IsChromeDoc(node->GetOwnerDoc())) {
+        NS_WARNING("Fix the caller!");
+      } else {
+        NS_ERROR("This is unsafe! Fix the caller!");
+      }
+    }
+  }
+
   if (aDOMEvent) {
     nsCOMPtr<nsIPrivateDOMEvent> privEvt(do_QueryInterface(aDOMEvent));
     if (privEvt) {
@@ -476,7 +531,7 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
 
   // If we have a PresContext, make sure it doesn't die before
   // event dispatching is finished.
-  nsCOMPtr<nsPresContext> kungFuDeathGrip(aPresContext);
+  nsRefPtr<nsPresContext> kungFuDeathGrip(aPresContext);
   ChainItemPool pool;
   NS_ENSURE_TRUE(pool.GetPool(), NS_ERROR_OUT_OF_MEMORY);
 
@@ -501,9 +556,7 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
     //     which are dispatched to |window| but have document as their target.
     //
     // Make sure that the event target points to the right object.
-    nsCOMPtr<nsPIDOMEventTarget> t = do_QueryInterface(aEvent->target);
-    NS_ENSURE_STATE(t);
-    aEvent->target = t->GetTargetForEventTargetChain();
+    aEvent->target = aEvent->target->GetTargetForEventTargetChain();
     NS_ENSURE_STATE(aEvent->target);
   }
   aEvent->originalTarget = aEvent->target;
@@ -566,11 +619,13 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
       } else {
         // Event target chain is created. Handle the chain.
         nsEventChainPostVisitor postVisitor(preVisitor);
+        nsCxPusher pusher;
         rv = topEtci->HandleEventTargetChain(postVisitor,
                                              NS_EVENT_FLAG_BUBBLE |
                                              NS_EVENT_FLAG_CAPTURE,
                                              aCallback,
-                                             PR_TRUE);
+                                             PR_FALSE,
+                                             &pusher);
   
         preVisitor.mEventStatus = postVisitor.mEventStatus;
         // If the DOM event was created during event flow.
@@ -687,6 +742,10 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
       return NS_NewDOMSVGZoomEvent(aDOMEvent, aPresContext,
                                    static_cast<nsGUIEvent*>(aEvent));
 #endif // MOZ_SVG
+#ifdef MOZ_SMIL
+    case NS_SMIL_TIME_EVENT:
+      return NS_NewDOMTimeEvent(aDOMEvent, aPresContext, aEvent);
+#endif // MOZ_SMIL
 
     case NS_COMMAND_EVENT:
       return NS_NewDOMCommandEvent(aDOMEvent, aPresContext,
@@ -694,6 +753,12 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
     case NS_SIMPLE_GESTURE_EVENT:
       return NS_NewDOMSimpleGestureEvent(aDOMEvent, aPresContext,
                                          static_cast<nsSimpleGestureEvent*>(aEvent));
+    case NS_MOZTOUCH_EVENT:
+      return NS_NewDOMMozTouchEvent(aDOMEvent, aPresContext,
+                                    static_cast<nsMozTouchEvent*>(aEvent));
+    case NS_TRANSITION_EVENT:
+      return NS_NewDOMTransitionEvent(aDOMEvent, aPresContext,
+                                      static_cast<nsTransitionEvent*>(aEvent));
     }
 
     // For all other types of events, create a vanilla event object.
@@ -739,6 +804,11 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
       aEventType.LowerCaseEqualsLiteral("svgzoomevents"))
     return NS_NewDOMSVGZoomEvent(aDOMEvent, aPresContext, nsnull);
 #endif // MOZ_SVG
+#ifdef MOZ_SMIL
+  if (aEventType.LowerCaseEqualsLiteral("timeevent") ||
+      aEventType.LowerCaseEqualsLiteral("timeevents"))
+    return NS_NewDOMTimeEvent(aDOMEvent, aPresContext, nsnull);
+#endif // MOZ_SMIL
   if (aEventType.LowerCaseEqualsLiteral("xulcommandevent") ||
       aEventType.LowerCaseEqualsLiteral("xulcommandevents"))
     return NS_NewDOMXULCommandEvent(aDOMEvent, aPresContext, nsnull);
@@ -760,8 +830,20 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
     return NS_NewDOMBeforeUnloadEvent(aDOMEvent, aPresContext, nsnull);
   if (aEventType.LowerCaseEqualsLiteral("pagetransition"))
     return NS_NewDOMPageTransitionEvent(aDOMEvent, aPresContext, nsnull);
+  if (aEventType.LowerCaseEqualsLiteral("moztouchevent"))
+    return NS_NewDOMMozTouchEvent(aDOMEvent, aPresContext, nsnull);
   if (aEventType.LowerCaseEqualsLiteral("scrollareaevent"))
     return NS_NewDOMScrollAreaEvent(aDOMEvent, aPresContext, nsnull);
+  // FIXME: Should get spec to say what the right string is here!  This
+  // is probably wrong!
+  if (aEventType.LowerCaseEqualsLiteral("transitionevent"))
+    return NS_NewDOMTransitionEvent(aDOMEvent, aPresContext, nsnull);
+  if (aEventType.LowerCaseEqualsLiteral("popstateevent"))
+    return NS_NewDOMPopStateEvent(aDOMEvent, aPresContext, nsnull);
+  if (aEventType.LowerCaseEqualsLiteral("mozaudioavailableevent"))
+    return NS_NewDOMAudioAvailableEvent(aDOMEvent, aPresContext, nsnull);
+  if (aEventType.LowerCaseEqualsLiteral("closeevent"))
+    return NS_NewDOMCloseEvent(aDOMEvent, aPresContext, nsnull);
 
   return NS_ERROR_DOM_NOT_SUPPORTED_ERR;
 }
