@@ -36,32 +36,39 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.6.4.3-2.js';
 
 /**
-   File Name:          15.6.4.3-2.js
-   ECMA Section:       15.6.4.3 Boolean.prototype.valueOf()
-   Description:        Returns this boolean value.
+   File Name:          15.6.3.1-4.js
+   ECMA Section:       15.6.3.1 Properties of the Boolean Prototype Object
 
-   The valueOf function is not generic; it generates
-   a runtime error if its this value is not a Boolean
-   object.  Therefore it cannot be transferred to other
-   kinds of objects for use as a method.
+   Description:        The initial value of Boolean.prototype is the built-in
+   Boolean prototype object (15.6.4).
+
+   The property shall have the attributes [DontEnum,
+   DontDelete, ReadOnly ].
+
+   This tests the ReadOnly property of Boolean.prototype
 
    Author:             christine@netscape.com
-   Date:               june 27, 1997
-*/
+   Date:               30 september 1997
 
-var SECTION = "15.6.4.3-2";
+*/
+var SECTION = "15.6.3.1-4";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Boolean.prototype.valueOf()";
+var TITLE   = "Boolean.prototype"
+  writeHeaderToLog( SECTION + TITLE );
 
-writeHeaderToLog( SECTION + " "+ TITLE);
+var BOOL_PROTO = Boolean.prototype;
 
+new TestCase( SECTION,
+	      "var BOOL_PROTO = Boolean.prototype; Boolean.prototype=null; Boolean.prototype == BOOL_PROTO",
+	      true,
+	      eval("var BOOL_PROTO = Boolean.prototype; Boolean.prototype=null; Boolean.prototype == BOOL_PROTO") );
 
-new TestCase( SECTION, "valof=Boolean.prototype.valueOf; x=new Boolean(); x.valueOf=valof;x.valueOf()", false, eval("valof=Boolean.prototype.valueOf; x=new Boolean(); x.valueOf=valof;x.valueOf()") );
-   
-new TestCase( SECTION, "valof=Boolean.prototype.valueOf; x=new Boolean(true); x.valueOf=valof;x.valueOf()", true, eval("valof=Boolean.prototype.valueOf; x=new Boolean(true); x.valueOf=valof;x.valueOf()") );
+new TestCase( SECTION,
+	      "var BOOL_PROTO = Boolean.prototype; Boolean.prototype=null; Boolean.prototype == null",
+	      false,
+	      eval("var BOOL_PROTO = Boolean.prototype; Boolean.prototype=null; Boolean.prototype == null") );
 
 test();
