@@ -38,39 +38,47 @@
 
 
 /**
-   File Name:          11.4.8.js
-   ECMA Section:       11.4.8 Bitwise NOT Operator
-   Description:        flip bits up to 32 bits
-   no special cases
+   File Name:          11.10-3.js
+   ECMA Section:       11.10-3 Binary Bitwise Operators:  ^
+   Description:
+   Semantics
+
+   The production A : A @ B, where @ is one of the bitwise operators in the
+   productions &, ^, | , is evaluated as follows:
+
+   1.  Evaluate A.
+   2.  Call GetValue(Result(1)).
+   3.  Evaluate B.
+   4.  Call GetValue(Result(3)).
+   5.  Call ToInt32(Result(2)).
+   6.  Call ToInt32(Result(4)).
+   7.  Apply the bitwise operator @ to Result(5) and Result(6). The result is
+   a signed 32 bit integer.
+   8.  Return Result(7).
+
    Author:             christine@netscape.com
-   Date:               7 july 1997
-
-   Data File Fields:
-   VALUE           value passed as an argument to the ~ operator
-   E_RESULT        expected return value of ~ VALUE;
-
-   Static variables:
-   none
-
+   Date:               12 november 1997
 */
-
-var SECTION = "11.4.8";
+var SECTION = "11.10-3";
 var VERSION = "ECMA_1";
 startTest();
 
-writeHeaderToLog( SECTION + " Bitwise Not operator");
+writeHeaderToLog( SECTION + " Binary Bitwise Operators:  ^");
 
-for ( var i = 0; i < 35; i++ ) {
-  var p = Math.pow(2,i);
+var shiftexp = 0;
+var addexp = 0;
 
-  new TestCase( SECTION, "~"+p,   Not(p),     ~p );
+for ( shiftpow = 0; shiftpow < 33; shiftpow++ ) {
+  shiftexp += Math.pow( 2, shiftpow );
 
-}
-for ( i = 0; i < 35; i++ ) {
-  var p = -Math.pow(2,i);
-
-  new TestCase( SECTION, "~"+p,   Not(p),     ~p );
+  for ( addpow = 0; addpow < 33; addpow++ ) {
+    addexp += Math.pow(2, addpow);
 
+    new TestCase( SECTION,
+		  shiftexp + " ^ " + addexp,
+		  Xor( shiftexp, addexp ),
+		  shiftexp ^ addexp );
+  }
 }
 
 test();
@@ -140,7 +148,7 @@ function Mask( b, n ) {
 }
 function ToUint32BitString( n ) {
   var b = "";
-  for ( var p = 31; p >=0; p-- ) {
+  for ( p = 31; p >=0; p-- ) {
     if ( n >= Math.pow(2,p) ) {
       b += "1";
       n -= Math.pow(2,p);
@@ -156,7 +164,7 @@ function ToInt32BitString( n ) {
 
   b += ( sign == 1 ) ? "0" : "1";
 
-  for ( var p = 30; p >=0; p-- ) {
+  for ( p = 30; p >=0; p-- ) {
     if ( (sign == 1 ) ? sign * n >= Math.pow(2,p) : sign * n > Math.pow(2,p) ) {
       b += ( sign == 1 ) ? "1" : "0";
       n -= sign * Math.pow( 2, p );
@@ -188,27 +196,72 @@ function ToInt32Decimal( bin ) {
 function ToUint32Decimal( bin ) {
   var r = 0;
 
-  for ( var l = bin.length; l < 32; l++ ) {
+  for ( l = bin.length; l < 32; l++ ) {
     bin = "0" + bin;
   }
 
-  for ( var j = 0; j < 31; j++ ) {
+  for ( j = 0; j < 31; j++ ) {
     r += Math.pow( 2, j ) * Number(bin.charAt(31-j));
+
   }
 
   return r;
 }
-function Not( n ) {
-  n = ToInt32(n);
-  n = ToInt32BitString(n);
+function And( s, a ) {
+  s = ToInt32( s );
+  a = ToInt32( a );
 
-  var r = "";
+  var bs = ToInt32BitString( s );
+  var ba = ToInt32BitString( a );
 
-  for( var l = 0; l < n.length; l++  ) {
-    r += ( n.charAt(l) == "0" ) ? "1" : "0";
+  var result = "";
+
+  for ( var bit = 0; bit < bs.length; bit++ ) {
+    if ( bs.charAt(bit) == "1" && ba.charAt(bit) == "1" ) {
+      result += "1";
+    } else {
+      result += "0";
+    }
+  }
+  return ToInt32Decimal(result);
+}
+function Xor( s, a ) {
+  s = ToInt32( s );
+  a = ToInt32( a );
+
+  var bs = ToInt32BitString( s );
+  var ba = ToInt32BitString( a );
+
+  var result = "";
+
+  for ( var bit = 0; bit < bs.length; bit++ ) {
+    if ( (bs.charAt(bit) == "1" && ba.charAt(bit) == "0") ||
+	 (bs.charAt(bit) == "0" && ba.charAt(bit) == "1")
+      ) {
+      result += "1";
+    } else {
+      result += "0";
+    }
   }
 
-  n = ToInt32Decimal(r);
+  return ToInt32Decimal(result);
+}
+function Or( s, a ) {
+  s = ToInt32( s );
+  a = ToInt32( a );
 
-  return n;
+  var bs = ToInt32BitString( s );
+  var ba = ToInt32BitString( a );
+
+  var result = "";
+
+  for ( var bit = 0; bit < bs.length; bit++ ) {
+    if ( bs.charAt(bit) == "1" || ba.charAt(bit) == "1" ) {
+      result += "1";
+    } else {
+      result += "0";
+    }
+  }
+
+  return ToInt32Decimal(result);
 }
