@@ -36,10 +36,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '7.4.2-16-n.js';
 
 /**
-   File Name:          7.4.2-16-n.js
+   File Name:          7.4.2-9-n.js
    ECMA Section:       7.4.2
 
    Description:
@@ -62,16 +61,16 @@ gTestfile = '7.4.2-16-n.js';
    Date:               12 november 1997
 
 */
-var SECTION = "7.4.1-16-n";
+var SECTION = "7.4.1-9-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Keywords";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var with = true";
+DESCRIPTION = "var delete = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var with = true",     "error",    eval("var with = true") );
+new TestCase( SECTION,  "var delete = true",     "error",    eval("var delete = true") );
 
 test();
