@@ -38,32 +38,25 @@
 
 
 /**
-   File Name:          15.8.1.6-2.js
-   ECMA Section:       15.8.1.6.js
+   File Name:          15.8.1.8-3.js
+   ECMA Section:       15.8.1.8.js
    Description:        All value properties of the Math object should have
    the attributes [DontEnum, DontDelete, ReadOnly]
 
-   this test checks the DontDelete attribute of Math.PI
+   this test checks the DontDelete attribute of Math.SQRT2
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
-
-var SECTION = "15.8.1.6-2";
+var SECTION = "15.8.1.8-3";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Math.PI";
 
-writeHeaderToLog( SECTION + " "+ TITLE);
+writeHeaderToLog( SECTION + " Math.SQRT2:  DontDelete");
 
 new TestCase( SECTION,
-	      "delete Math.PI; Math.PI",   
-	      3.1415926535897923846, 
-	      eval("delete Math.PI; Math.PI") );
-
-new TestCase( SECTION,
-	      "delete Math.PI; Math.PI", 
-	      false,   
-              eval("delete Math.PI") );
+	      "delete Math.SQRT2",
+	      false,    
+	      eval("delete Math.SQRT2") );
 
 test();
