@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Bertrand Le Roy
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,14 +36,52 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 310425;
-var summary = 'Array.indexOf/lastIndexOf edge cases';
+var BUGNUMBER = 350256;
+var summary = 'Array.apply maximum arguments: 2^19-1024';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
- 
-expect = -1;
-actual = Array(1).indexOf(1);
-reportCompare(expect, actual, summary);
+
+//-----------------------------------------------------------------------------
+test(Math.pow(2, 19)-1024);
+//-----------------------------------------------------------------------------
+
+function test(length)
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+
+  try
+  {
+
+    var a = new Array();
+    a[length - 2] = 'length-2';
+    a[length - 1] = 'length-1';
+
+    var b = Array.apply(null, a);
+
+    expect = length + ',length-2,length-1';
+    actual = b.length + "," + b[length - 2] + "," + b[length - 1];
+    reportCompare(expect, actual, summary);
+
+    function f() {
+      return arguments.length + "," + arguments[length - 2] + "," +
+        arguments[length - 1];
+    }
+
+    expect = length + ',length-2,length-1';
+    actual = f.apply(null, a);
+
+  }
+  catch(ex)
+  {
+    expect = 'InternalError: script stack space quota is exhausted';
+    actual = ex + '';
+    print(actual);
+  }
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
+}
