@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-465460-08.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 465460;
-var summary = 'TM: valueOf in a loop: do not assert';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
@@ -52,12 +51,13 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
 
-  for each (let _ in [{}, {}, null, null, null, null]) { }
+// ------- Comment #121 From Gary Kwong [:nth10sd]
 
-  jit(false);
+// without -j
+  x = function()x;
+
+// Assertion failure: !(pn->pn_dflags & flag), at ../jsparse.h:651
 
   reportCompare(expect, actual, summary);
 
